/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package core;

import java.io.PrintWriter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 *
 * @author pchr
 */
public class Domain {
    
    private Map<Integer,Node> theNodes=new HashMap<Integer,Node>();
    private Map<Integer,Material> theMaterials=new HashMap<Integer,Material>();
    private Map<Integer,Element> theElements=new HashMap<Integer,Element>();
    private Map<Integer,Section> theSections=new HashMap<Integer,Section>();
    private Map<Integer,Load> theLoads = new HashMap<Integer,Load>();
    private Map<Integer,Constraint> theConstraints = new HashMap<Integer,Constraint>();
    private Soe soe;
    
    public Domain(){
        (new Node(-10,0.)).init();
    }
    
    public Node getNode(int id){
        return this.theNodes.get(id);
    }
    
    public Element getElement(int id){
        return this.theElements.get(id);
    }
    
    public Material getMaterial(int id){
        return this.theMaterials.get(id);
    }
    
    public Section getSection(int id){
        return this.theSections.get(id);
    }
    
    public int getNodeCount(){
        return this.theNodes.size();
    }
    
    public int getElementCount(){
        return this.theElements.size();
    }
    
    public Map<Integer,Node> getNodes(){
        return this.theNodes;
    }
    
    public Map<Integer,Element> getElements(){
        return this.theElements;
    }
    
    public Map<Integer,Material> getMaterials(){
        return this.theMaterials;
    }
    
    public Map<Integer,Section> getSections(){
        return this.theSections;
    }
    
    public Map<Integer,Load> getLoads(){
        return this.theLoads;
    }
    
    public Map<Integer,Constraint> getConstraints(){
        return this.theConstraints;
    }
    
    public void putNode(Node aNode){
        this.theNodes.put(aNode.getId(), aNode);
    }
    
    public void putElement(Element anElement){
        this.theElements.put(anElement.getId(), anElement);
    }
    
    public void putMaterial(Material aMaterial){
        this.theMaterials.put(aMaterial.getId(), aMaterial);
    }
    
    public void putSection(Section aSection){
        this.theSections.put(aSection.getId(), aSection);
    }
    
    public void putLoad(Load aLoad){
        this.theLoads.put(aLoad.getId(), aLoad);
    }
    
    public void putConstraint(Constraint aConstraint){
        this.theConstraints.put(aConstraint.getId(), aConstraint);
    }
    
    public void setNodes(Map<Integer,Node> someNodes){
        this.theNodes=someNodes;
    }
    
    public void setElements(Map<Integer,Element> someElements){
        this.theElements=someElements;
    }
    
    public void setMaterials(Map<Integer,Material> someMaterials){
        this.theMaterials=someMaterials;
    }
    
    public void setLoads(Map<Integer,Load> someLoads){
        this.theLoads=someLoads;
    }
    
    public void setConstraints(Map<Integer,Constraint> someConstraints){
        this.theConstraints=someConstraints;
    }
    
    public void setSections(Map<Integer,Section> someSections){
        this.theSections=someSections;
    }
    
    public boolean NodeInUse(int node_id){
        boolean answer=false;
        for(Iterator<Element> it=this.theElements.values().iterator(); it.hasNext();){
            Element theElement = it.next();
            if(theElement.getN1().getId()==node_id){answer=true;}
            if(theElement.getN2().getId()==node_id){answer=true;}
        }
        for(Iterator<Load> it=this.theLoads.values().iterator(); it.hasNext();){
            Load theLoad = it.next();
            if(theLoad.getNode().getId()==node_id){answer=true;}
        }
        for(Iterator<Constraint> it=this.theConstraints.values().iterator(); it.hasNext();){
            Constraint theConstraint = it.next();
            if(theConstraint.getNode().getId()==node_id){answer=true;}
        }
        return answer;
    }
    
    public boolean NodeExistence(int node_id){
        boolean answer=false;
        for(Iterator<Node> it=this.theNodes.values().iterator(); it.hasNext();){
            Node theNode = it.next();
            if(theNode.getId()==node_id){answer=true;}
        }
        
        return answer;
    }
    
    public boolean MaterialInUse(int mat_id){
        boolean answer=false;
        for(Iterator<Section> it=this.theSections.values().iterator(); it.hasNext();){
            Section theSection = it.next();
            if(theSection.getMaterial().getId()==mat_id){answer=true;}
        }
        return answer;
    }
    
    public boolean MaterialExistence(int mat_id){
        boolean answer=false;
        for(Iterator<Material> it=this.theMaterials.values().iterator(); it.hasNext();){
            Material theMaterial = it.next();
            if(theMaterial.getId()==mat_id){answer=true;}
        }
        
        return answer;
    }
    
    public boolean SectionInUse(int sec_id){
        boolean answer=false;
        for(Iterator<Element> it=this.theElements.values().iterator(); it.hasNext();){
            Element theElement = it.next();
            if(theElement.getSection().getId()==sec_id){answer=true;}
        }
        return answer;
    }
    
    public boolean SectionExistence(int sec_id){
        boolean answer=false;
        for(Iterator<Section> it=this.theSections.values().iterator(); it.hasNext();){
            Section theSection = it.next();
            if(theSection.getId()==sec_id){answer=true;}
        }
        
        return answer;
    }
    
    public boolean ElementExistence(int elem_id){
        boolean answer=false;
        for(Iterator<Element> it=this.theElements.values().iterator(); it.hasNext();){
            Element theElement = it.next();
            if(theElement.getId()==elem_id){answer=true;}
        }
        
        return answer;
    }
    
    public double getMinX(){
        double minx=0.;
        double xcur=0.;
        int fi=0;
        for(Iterator<Node> it=this.theNodes.values().iterator(); it.hasNext();){
            ++fi;
            Node theNode = it.next();
            if(fi==1){minx=theNode.getX();}
            xcur=theNode.getX();
            if(xcur<minx){minx=xcur;}
            
        }
        return minx;
    }
    
    public double getMaxX(){
        double maxx=0.;
        double xcur=0.;
        int fi=0;
        for(Iterator<Node> it=this.theNodes.values().iterator(); it.hasNext();){
            ++fi;
            Node theNode = it.next();
            if(fi==1){maxx=theNode.getX();}
            xcur=theNode.getX();
            if(xcur>maxx){maxx=xcur;}
            
        }
        return maxx;
    }
    
    public double getMinU(){
        double min=0.;
        double cur=0.;
        int fi=0;
        for(Iterator<Element> it=this.theElements.values().iterator(); it.hasNext();){
            ++fi;
            Element theElement = it.next();
            if(fi==1){min=theElement.getU()[0];}
            cur=theElement.getU()[0];
            if(cur<min){min=cur;}
            cur=theElement.getU()[1];
            if(cur<min){min=cur;}
            
        }
        return min;
    }
    
    public double getMaxU(){
        double min=0.;
        double cur=0.;
        int fi=0;
        for(Iterator<Element> it=this.theElements.values().iterator(); it.hasNext();){
            Element theElement = it.next();
            if(fi==1){min=theElement.getU()[0];}
            cur=theElement.getU()[0];
            if(cur>min){min=cur;}
            cur=theElement.getU()[1];
            if(cur>min){min=cur;}
            
        }
        return min;
    }
    
     public double getMinUi(){
        double min=0.;
        double cur=0.;
        int fi=0;
        for(Iterator<Element> it=this.theElements.values().iterator(); it.hasNext();){
            ++fi;
            Element theElement = it.next();
            if(fi==1){min=theElement.getUi()[0];}
            cur=theElement.getUi()[0];
            if(cur<min){min=cur;}
            
        }
        return min;
    }
    
    public double getMaxUi(){
        double min=0.;
        double cur=0.;
        int fi=0;
        for(Iterator<Element> it=this.theElements.values().iterator(); it.hasNext();){
            ++fi;
            Element theElement = it.next();
            if(fi==1){min=theElement.getUi()[0];}
            cur=theElement.getUi()[0];
            if(cur>min){min=cur;}
            
        }
        return min;
    }
    
    public double getMinF(){
        double min=0.;
        double cur=0.;
        int fi=0;
        for(Iterator<Element> it=this.theElements.values().iterator(); it.hasNext();){
            ++fi;
            Element theElement = it.next();
            if(fi==1){min=theElement.getF()[0];}
            cur=theElement.getF()[0];
            if(cur<min){min=cur;}
            cur=theElement.getF()[1];
            if(cur<min){min=cur;}
            
        }
        return min;
    }
    
    public double getMaxF(){
        double min=0.;
        double cur=0.;
        int fi=0;
        for(Iterator<Element> it=this.theElements.values().iterator(); it.hasNext();){
            ++fi;
            Element theElement = it.next();
            if(fi==1){min=theElement.getF()[0];}
            cur=theElement.getF()[0];
            if(cur>min){min=cur;}
            cur=theElement.getF()[1];
            if(cur>min){min=cur;}
            
        }
        return min;
    }
    
    public double getMinS(){
        double min=0.;
        double cur=0.;
        int fi=0;
        for(Iterator<Element> it=this.theElements.values().iterator(); it.hasNext();){
            ++fi;
            Element theElement = it.next();
            if(fi==1){min=theElement.getS()[0];}
            cur=theElement.getS()[0];
            if(cur<min){min=cur;}
            cur=theElement.getS()[1];
            if(cur<min){min=cur;}
            
        }
        return min;
    }
    
    public double getMaxS(){
        double min=0.;
        double cur=0.;
        int fi=0;
        for(Iterator<Element> it=this.theElements.values().iterator(); it.hasNext();){
            ++fi;
            Element theElement = it.next();
            if(fi==1){min=theElement.getS()[0];}
            cur=theElement.getS()[0];
            if(cur>min){min=cur;}
            cur=theElement.getS()[1];
            if(cur>min){min=cur;}
            
        }
        return min;
    }
    
    public void analyse(){
        int ndofs=new Node(-1,0).getNdofs();
        this.soe=new Soe();
        this.soe.init(ndofs);
        
        int[] eft = new int[2];
        for(Iterator<Element> it=this.theElements.values().iterator(); it.hasNext();){
            Element theElement = it.next();
            eft=theElement.getFTable();
            for(int i=0; i<2; i++){
                for(int j=0; j<2; j++){
                    soe.addA(eft[i]-1, eft[j]-1, theElement.getK().get(i, j));
                }
            }
            
        }
        
        eft = new int[1];
        for(Iterator<Constraint> it=this.theConstraints.values().iterator(); it.hasNext();){
            Constraint theConstraint = it.next();
            eft=theConstraint.getFTable();
            soe.addA(eft[0]-1, eft[0]-1, theConstraint.getK().get(0, 0));
            soe.addB(eft[0]-1, (theConstraint.getK().get(0, 0))*theConstraint.getConstraintValue() );
        }
        
        for(Iterator<Load> it=this.theLoads.values().iterator(); it.hasNext();){
            Load theLoad = it.next();
            eft=theLoad.getFTable();
            soe.addB(eft[0]-1, theLoad.getLoadValue() );
        }
        
        /////////////////////////////
        if(this.theConstraints.size()>0){
            this.soe.solve();
        }else{
            this.soe.solveSVD();
        }
        ////////////////////////////
        
        for(Iterator<Node> it=this.theNodes.values().iterator(); it.hasNext();){
            Node theNode = it.next();
            theNode.updateNode(soe.getX());
            
            //System.out.print(theNode.getId()+" ");
            //System.out.println(theNode.getU()[0]);
        }
        
        /*
        for(Iterator<Element> it=this.theElements.values().iterator(); it.hasNext();){
            Element theElement = it.next();
            System.out.print(theElement.getId()+" ");
            System.out.print(theElement.getF()[0]+" ");
            System.out.println(theElement.getF()[1]);
        }
        */
    }
    
    public void exportFile(PrintWriter FileToWrite){
        for(Iterator<Node> it=this.theNodes.values().iterator(); it.hasNext();){
            Node theNode = it.next();
            FileToWrite.print("κόμβος ");
            FileToWrite.print(theNode.getId()+" "+theNode.getX());
            FileToWrite.println();
        }
        FileToWrite.println();
        
        for(Iterator<Material> it=this.theMaterials.values().iterator(); it.hasNext();){
            Material theMaterial = it.next();
            FileToWrite.print("υλικό ");
            FileToWrite.print(theMaterial.getId()+" "+theMaterial.getGamma());
            FileToWrite.println();
        }
        FileToWrite.println();
        
        for(Iterator<Section> it=this.theSections.values().iterator(); it.hasNext();){
            Section theSection = it.next();
            FileToWrite.print("διατομή ");
            FileToWrite.print(theSection.getId()+" "+theSection.getIp()+" "+theSection.getMaterial().getId());
            FileToWrite.println();
        }
        FileToWrite.println();
        
        for(Iterator<Element> it=this.theElements.values().iterator(); it.hasNext();){
            Element theElement = it.next();
            FileToWrite.print("στοιχείο ");
            FileToWrite.print(theElement.getId()+" "+theElement.getN1().getId()+" "+theElement.getN2().getId()+" "+theElement.getSection().getId());
            FileToWrite.println();
        }
        FileToWrite.println();
        
        for(Iterator<Load> it=this.theLoads.values().iterator(); it.hasNext();){
            Load theLoad = it.next();
            FileToWrite.print("δύναμη ");
            FileToWrite.print(theLoad.getNode().getId()+" "+theLoad.getLoadValue());
            FileToWrite.println();
        }
        FileToWrite.println();
        
        for(Iterator<Constraint> it=this.theConstraints.values().iterator(); it.hasNext();){
            Constraint theConstraint = it.next();
            FileToWrite.print("μετατόπιση ");
            FileToWrite.print(theConstraint.getNode().getId()+" "+theConstraint.getConstraintValue());
            FileToWrite.println();
        }
        FileToWrite.println();
        
        FileToWrite.close();
    }
    
    public void exportResults(PrintWriter FileToWrite){
        for(Iterator<Element> it=this.theElements.values().iterator(); it.hasNext();){
            Element theElement = it.next();
            FileToWrite.print("στοιχείο                      :");FileToWrite.println(theElement.getId());
            FileToWrite.print("μετατοπίσεις                  :");FileToWrite.print(theElement.getU()[0]);FileToWrite.print(" ");FileToWrite.println(theElement.getU()[1]);
            FileToWrite.print("δυνάμεις                      :");FileToWrite.print(theElement.getF()[0]);FileToWrite.print(" ");FileToWrite.println(theElement.getF()[1]);
            FileToWrite.print("επιμήκυνση στοιχείου          :");FileToWrite.println(theElement.getUi()[0]);
            FileToWrite.print("ορθές τάσεις                  :");FileToWrite.print(theElement.getS()[0]);FileToWrite.print(" ");FileToWrite.println(theElement.getS()[1]);
            FileToWrite.println();FileToWrite.println();
        }
        FileToWrite.println();
        
        
        FileToWrite.close();
    }

}
